% TODO - add description, also to properties etc.
classdef recSettings < matlab.mixin.Copyable
    properties
        input (1, 1) struct;
        output (1, 1) struct;
        tasks (:, 1) cell;
        rec (1, 1) struct;
    end
    methods
        
        function obj = recSettings() % constructor
            % inputs
            obj.input(1).path = '';
            obj.input(1).name = '';
            obj.input(1).format = '';
            obj.input(1).metapath = '';
            obj.input(1).metaname = '';
            obj.input(1).metaformat = '';
            % outputs
            obj.output(1).projFlag = false;
            obj.output(1).recFlag = false;
            obj.output(1).path = '';
            obj.output(1).projname = '';
            obj.output(1).recname = '';
            obj.output(1).projformat = '';
            obj.output(1).recformat = '';
            % projection processing
            obj.tasks = {};
            % reconstruction
            obj.rec(1).recFlag = false;
            obj.rec(1).toolbox = '';
            obj.rec(1).algorithm = '';
            obj.rec(1).algoSettings = struct;
        end
        
        function addInput(obj, fullname) 
            % add path to input file
            % parse inputs - contains only path as char vector
            [filepath, name, format] = fileparts(fullname);
            % TODO - check whether file exists
            if ~isempty(filepath) || ~isempty(name) || ~isempty(format)
                filepath = [filepath, '\']; % add backslash to filepath
                if ~strcmp(format, '.raw')
                    % throw warning, but continue
                    warning('Chosen file is not in *.raw format. Only *.raw is supported currently.')
                end
                % write the file name and path into the structure
                obj.input.path = filepath;
                obj.input.name = name;
                obj.input.format = format;
                
                % Check for a *.ctm file, and if metapath isn't set already
                
                if exist([filepath, name, '.ctm'], 'file') == 2
                    % if a file has been found, add it
                    obj.input.metapath = filepath;
                    obj.input.metaname = name;
                    obj.input.metaformat = '.ctm';
                elseif exist([filepath, name, '.ipro'], 'file') == 2
                    obj.input.metapath = filepath;
                    obj.input.metaname = name;
                    obj.input.metaformat = '.ipro';
                elseif isempty(obj.input.metaname) % only checking 1 of the 3 fields
                   % if no prior metadata path and no file was found
                    warningMessage = ['A *.ctm file associated with the chosen input ', ...
                        'was not found in the input directory. Most functions will not ', ...
                        'work without a *.ctm file. Only metadata with the suffix .ctm ', ...
                        'are supported currently'];
                    warning(warningMessage)
                end
            else
                warning('Invalid or no file chosen, no changes were made.')
            end
        end
        % TODO - output messages as log or as warning in the end,
            % depending on nargout
        % TODO - add support for other formats in the future?
        % TODO - check if everything works as expected
        
        function addMeta(obj, fullname)
            % If I want to change only the metafile
            % TODO - parse inputs - contains only path as char vector
            
            % Add/Change path to metafile
            [filepath, name, ext] = fileparts(fullname);
            if ~isempty(filepath) || ~isempty(name) || ~isempty(ext)
                filepath = [filepath, '\']; % add backslash to filepath
                if ~strcmp(ext, '.ctm')
                    % throw warning, but continue
                    warning('Caution: chosen metafile is not in *.ctm format.')
                end
                if ~isempty(obj.input.metapath)
                    warning('Overwriting previous metafile path')
                end
                obj.input.metapath = filepath;
                obj.input.metaname = name;
                obj.input.metaformat = ext;
            else
                warning('Invalid or no file chosen, no changes were made.')
            end
            % TODO - output messages as log or as warning in the end,
            % depending on nargout
            
        end
        
        function rmInput(obj)
            % utility function for deleting input info
            
            % set all to blank
            obj.input.path = '';
            obj.input.name = '';
            obj.input.format = '';
            obj.input.metapath = '';
            obj.input.metaname = '';
            obj.input.metaformat = '';
        end
        
        
        
        function addOutput(obj, directory, name, projformat, recformat)
            % add output path, specify output filetypes and what to save
            
            % TODO  - update description
            
            % TODO - check inputs
            
            % set name and format for saving projection data
            if exist('projformat', 'var') && ~isempty(projformat)
                obj.output.projname = ['proj_' name];
                if ~strcmp(projformat(1), '.')
                    projformat = ['.', projformat];
                end
                obj.output.projFlag = true;
                obj.output.projformat = projformat;
            else
                obj.output.projname = [];
                obj.output.projFlag = false;
                obj.output.projformat = [];
            end
            % set name and format for saving reconstruction data
            if exist('recformat', 'var') && ~isempty(recformat)
                obj.output.recname = ['rec_' name];
                if ~strcmp(recformat(1), '.')
                    recformat = ['.', recformat];
                end
                obj.output.recFlag = true;
                obj.output.recformat = recformat;
            else
                obj.output.recname = [];
                obj.output.recFlag = false;
                obj.output.recformat = [];
            end
            % set directory path - only if proj or recon path is set
            if obj.output.projFlag || obj.output.recFlag
                if ~strcmp(directory(end), '\')
                    directory = [directory, '\'];
                end
                obj.output.path = directory;
            else
                obj.output.path = [];
            end
            
        end
        
        function rmOutput(obj)
            % utility function for deleting output info
            
            % set everything to blank
            obj.output.projFlag = false;
            obj.output.recFlag = false;
            obj.output.path = '';
            obj.output.projname = '';
            obj.output.recname = '';
            obj.output.projformat = '';
            obj.output.recformat = '';
        end
        
        
        
        function addTask(obj, funcString, varargin)
            % for adding specifications for projection processing
            % funHandle can be a string or a function handle - will be
            % converted to function handle
            % varargin can be either name-value pairs or a structure
            
            % parse inputs - I don't know what will be passed as varargin,
            % so I will just keep unmatche inputs and save them
            p = inputParser;
            p.KeepUnmatched = true;
            parse(p,varargin{:})
            
            % if funHandle is string, convert it to function handle
            if isa(funcString, 'function_handle')
                funcString = func2str(funcString);
            end
            % get current number of tasks
            numtasks = length(obj.tasks);
            % append new task
            obj.tasks{numtasks + 1}.name = funcString;
            obj.tasks{numtasks + 1}.inputs = p.Unmatched;
        end

        function rmTask(obj, indexToRemove)
            % TODO - check inputs
            
            numTasks = numel(obj.tasks);
            % check input
            validateattributes(indexToRemove, {'numeric'}, ...
                               {'vector', 'integer', 'positive', ...
                               '<=', numTasks}, 'rmTask', 'indexToRemove')
                           
            for k = indexToRemove
                obj.tasks{k} = {}; % make selected indexes empty
            end
            obj.tasks = obj.tasks(~cellfun(@isempty, obj.tasks)); % remove empty indexes
            
        end
        % TODO - can probably be done better
        
        function rmAllTasks(obj)
            % utility function for deleting all task settings
            obj.tasks = {};
        end
        
        function taskList = listTasks(obj, includeInputs)
            % includeInputs - logical flag whether to include function 
            % inputs in the list
            
            % TODO: add description, comments
            
            if nargin < 2; includeInputs = false; end
            
            numTasks = numel(obj.tasks);
            
            % preallocate cell array
            taskList = cell(numTasks, 1);
            
            for k = 1:numTasks
                if includeInputs
                    inputs = [fieldnames(obj.tasks{k}.inputs), ...
                          struct2cell(obj.tasks{k}.inputs)];
                    inputs = cellfun(@num2str, inputs, ...
                                     'UniformOutput', false);
                    
                    taskList{k, 1} = [obj.tasks{k}.name, ','];
                    
                    for m = 1:( numel(inputs) / 2 )
                        taskList{k, 1} = sprintf('%s %s: %s,', ...
                            taskList{k, 1}, inputs{m, 1}, inputs{m, 2});
                    end
                    % remove last comma
                    taskList{k, 1} = taskList{k, 1}(1:(end-1));
                else
                    taskList{k, 1} = obj.tasks{k}.name;
                end
            end
        end
        
        function sortTasks(obj, order)
            
            % use validateattributes instead?
            if all( mod(order, 1) == 0) ... % all numbers are whole numbers
                    && ...
               all( order > 0 ) ... % all numbers are positive
                    && ...
               numel(obj.tasks) == numel(order) ... % if number of tasks stays the same
                    && ...
               numel(order) == numel( unique(order) ) ... % if no task is repeated
           
                % if everything is alright, then order the fields according
                % to the vector order
                obj.tasks = obj.tasks(order);
            else
                error(['Input vector must be composed entirely of' ...
                       ' positive integers, it must have the same' ...
                       ' number of elements as there are tasks in' ...
                       ' recSet.tasks and no number must be repeated.'])
            end
            
        end
        
        
        
        function addRec(obj, toolbox, algorithm, varargin)
            % for specifying reconstruction settings
            % TODO - check inputs
            
            % Parse inputs, leave unmatched
            p = inputParser;
            p.KeepUnmatched = true;
            parse(p,varargin{:})
            
            obj.rec.recFlag = true; % if everything is OK, set recFlag to 1
            obj.rec.toolbox = toolbox;
            obj.rec.algorithm = algorithm;
            obj.rec.algoSettings = p.Unmatched;
        end
        
        function rmRec(obj)
            % utility function to remove reconstruction settings
            % TODO - add better description
            obj.rec = struct;
            obj.rec.recFlag = false;
            obj.rec.toolbox = '';
            obj.rec.algorithm = '';
            obj.rec.algoSettings = struct;
        end
        
        function recInfo = getRecInfo(obj)
            % add info about recFlag
            recInfo = sprintf('%s %i', 'Reconstruction flag set to', ...
                              obj.rec.recFlag);
            % add info about toolbox and algorithm
            recInfo = sprintf('%s\n%s: %s\n%s: %s', recInfo, ...
                           'Reconstruction toolbox', obj.rec.toolbox, ...
                           'Reconstruction algorithm', obj.rec.algorithm);
            % add info about algorithm inputs
            recInfo = sprintf('%s\n%s', recInfo, 'Algorithm inputs:');
            algoInputNames = fieldnames(obj.rec.algoSettings);
            
            algoInputs = struct2cell(obj.rec.algoSettings);
            algoInputs = cellfun(@num2str, algoInputs, 'UniformOutput', false);
            
            for k = 1:numel(algoInputNames)
                recInfo = sprintf('%s\n\t%s: %s', recInfo, ...
                                  algoInputNames{k}, algoInputs{k});
            end
            
        end
        
        
        
        function saveSettings(obj, filename)
            % TODO - save as xml
            % save structure to xml file
            % save only tasks and rec
            
            % WARNING: ugly code ahead
            
            % create handle to num2str function
            n2s = str2func('num2str');
            % convert all attributes in tasks to char 
            tempTasks = obj.tasks; % first get the tasks cell array
            for k = 1:length(tempTasks) % for each cell, convert all to char
                % convert function handle to char
                tempTasks{k}.name = char(tempTasks{k}.name);
                % go through function inputs and convert to char
                if islogical(tempTasks{k}.inputs)
                    % convert logicals to 'TRUE'/'FALSE' text
                    if tempTasks{k}.inputs
                        tempTasks{k}.inputs = 'TRUE';
                    else
                        tempTasks{k}.inputs = 'FALSE';
                    end
                else % else convert to char
                    tempTasks{k}.inputs = structfun(n2s, tempTasks{k}.inputs, 'UniformOutput', false);
                end
                % Note: function names are already char
                % rename structure fields for struct2xml function
                tempTasks{k} = renameStructField(tempTasks{k}, 'inputs', 'Attributes');
                tempTasks{k} = renameStructField(tempTasks{k}, 'name', 'Text');
            end
            % write converted tasks to s
            s.reconstruction_settings.tasks = tempTasks;
            
            % get rec structure
            recon = obj.rec;
            fields = fieldnames(recon);
            for k = 1:length(fields)
                if islogical(recon.(fields{k}))
                    % convert logicals to 'TRUE'/'FALSE' text
                    if recon.(fields{k})
                        recon.(fields{k}) = 'TRUE';
                    else
                        recon.(fields{k}) = 'FALSE';
                    end
                elseif isstruct(recon.(fields{k})) % apply num2str to all structure fields
                    recon.(fields{k}) = structfun(n2s, recon.(fields{k}), 'UniformOutput', false);
                else % apply num2str
                    recon.(fields{k}) = num2str(recon.(fields{k}));
                end
            end
            % write converted reconstruction settings to s
            s.reconstruction_settings.rec = recon;
            
            % save s as filename.xml
            struct2xml(s, filename);
            % note - check help for struct2xml for info on required
            % structure format
            
        end
        % TODO - check if it works
        
        function loadSettings(obj, filename)
            % load structure from xml file
            
            % WARNING: ugly code ahead
            
            % read *.xml file, convert to Matlab structure
            settingsFile = xml2struct(filename);
            
            % load processing settings
            obj = loadTasks(obj, settingsFile);
            % load reconstruction settings
            obj = loadRec(obj, settingsFile);

            
        end
        % TODO - seems to work now, but review it again + think of a more elegant way to code it
        % TODO - why does an object w/ loaded settings have a slightly
        % larger size than the original, even though isequal(obj1, obj2) gives 1?
        
        function loadTasks(obj, settingsFile)
            % TODO - add description
            % settingsFile can be either the output of xml2struct (used in
            % loadSettings), or it can be the path to an .xml file
            
            % TODO - clear previous tasks?
            
            if ischar(settingsFile) % if a path to a file was supplied
                % read *.xml file, convert to Matlab structure
                settingsFile = xml2struct(settingsFile);
            end
            
            % WARNING: ugly code ahead
            
            % first clear any previous processing settings
            obj = obj.rmAllTasks;
            
            % get only part about tasks - if it exists
            if isfield(settingsFile.reconstruction_settings, 'tasks')
                
                tempTasks = settingsFile.reconstruction_settings.tasks;
                
                % go through all entries in tasks, convert char to
                % numeric/logical where applicable
                for k = 1:length(tempTasks)
                    % convert string to function handle
                    tempTasks{k}.Text = tempTasks{k}.Text;
                    
                    % get field names of attributes (if there are any attributes)
                    if isfield(tempTasks{k}, 'Attributes') % if there is such a field
                        fields = fieldnames(tempTasks{k}.Attributes);
                        % go entry by entry and check if they are numbers or not
                        for m = 1:length(fields)
                            % get current field's value and convert to double
                            pom = str2double(tempTasks{k}.Attributes.(fields{m}));
                            if isnan(pom) % if str2double returns NaN, write current field as string
                                % check if it is a logical value
                                if strcmpi(tempTasks{k}.Attributes.(fields{m}), 'TRUE') % case insensitive
                                    tempTasks{k}.Attributes.(fields{m}) = true;
                                elseif strcmpi(tempTasks{k}.Attributes.(fields{m}), 'FALSE')
                                    tempTasks{k}.Attributes.(fields{m}) = false;
                                end % else just leave it as it is
                            else % else write as double
                                tempTasks{k}.Attributes.(fields{m}) = pom;
                            end
                        end
                    else % if there are no fields, just save empty struct
                        tempTasks{k}.Attributes = struct;
                    end
                    % rename the fields back to 'name' and 'inputs'
                    tempTasks{k} = renameStructField(tempTasks{k}, 'Attributes', 'inputs');
                    tempTasks{k} = renameStructField(tempTasks{k}, 'Text', 'name');
                end
                % Write into parameters of object
                obj.tasks = tempTasks;
            end
            
            
        end
        % TODO - make code more readable
        
        function loadRec(obj, settingsFile)
            % TODO - add description
            % settingsFile can be either the output of xml2struct (used in
            % loadSettings), or it can be the path to an .xml file
            
            if ischar(settingsFile) % if a path to a file was supplied
                % read *.xml file, convert to Matlab structure
                settingsFile = xml2struct(settingsFile);
            end
            
            % WARNING: ugly code ahead
            
            % first clear any previous reconstruction settings
            obj = obj.rmRec;
            
            % get only part about reconstruction
            tempRec = settingsFile.reconstruction_settings.rec;
            % convert the main fields - recFlag, toolbox, algorithm
            
            % if reconstruction flag is set to true
            if strcmpi(tempRec.recFlag.Text, 'TRUE') % case insensitive
            	tempRec.recFlag = true;
                % if recFlag is true, also write toolbox and algorithm
                tempRec.toolbox = tempRec.toolbox.Text;
                tempRec.algorithm = tempRec.algorithm.Text;
                
                % get field names of algorithm settings (if there are any)
                tempSettings = tempRec.algoSettings;
                fields = fieldnames(tempSettings);
                if strcmpi(fields{1}, 'Text') % this means that algoSettings is supposed to be empty
                    % write an empty struct into algoSettings
                    tempSettings = struct;
                else
                    % go entry by entry and check if they are numbers or not
                    for k = 1:length(fields)
                        % get current field's value and convert to double
                        pom = str2double(tempSettings.(fields{k}).Text);
                        if isnan(pom) % if str2double returns NaN, write current field as string
                            % check if it is a logical value
                            if strcmpi(tempSettings.(fields{k}).Text, 'TRUE') % case insensitive
                                tempSettings.(fields{k}) = true;
                            elseif strcmpi(tempSettings.(fields{k}).Text, 'FALSE')
                                tempSettings.(fields{k}) = false;
                            else % else just write as string
                                tempSettings.(fields{k}) = tempSettings.(fields{k}).Text;
                            end
                        else % else write as double
                            tempSettings.(fields{k}) = pom;
                        end
                    end
                end
                tempRec.algoSettings = tempSettings;
                
                % Write into parameters of object
                % this will overwrite what was there previously
                obj.rec = tempRec;
            end
            
        end
        % TODO - make code more readable
        
   end
end

