function [report] = reconstructionReport(data)
% Creating text report with information about used reconstruction settings
% TODO - description

% time report was created
timestamp = datestr(clock, 'mmmm dd, yyyy at HH:MM:SS'); % TODO - better format
report = sprintf('%s %s\n\n', 'Reconstruction report created on', timestamp);

s = data.settings;

% Reconstruction algorithm and toolbox
report = sprintf('%s%s: %s\n%s: %s\n', report, ...
                 'Reconstruction toolbox', s.rec.toolbox, ...
                 'Reconstruction algorithm', s.rec.algorithm);

% Reconstruction settings used
if isfield(data.misc, 'recInputs') % created inside the individual reconstruction functions - TODO - change this?
    report = sprintf('%s%s\n', report, data.misc.recInputs);
end

% Reconstruction time
if isfield(data.misc, 'recTime')
    report = sprintf('%s%s %.2f %s\n', report, 'Reconstruction completed in', data.misc.recTime, 'seconds');
end

% dimensions of reconstruction volume
report = sprintf('%s%s\n\t%s %i\n\t%s %i\n\t%s %i\n', report, ...
    'Dimensions of reconstructed volume (voxels):', ...
    'x:', data.geom.reconWidth, 'y:', data.geom.reconWidth, ...
    'z:', data.geom.reconHeight);

% volume voxel size
report = sprintf('%s%s %.6f %s\n', report, ...
    'Voxel size:', data.geom.vox_size, 'micrometers');

% Input and output path

report = sprintf('%s%s:  %s%s%s\n',  report, 'Input projection file',...
    s.input.path, s.input.name, s.input.format);

report = sprintf('%s%s:  %s%s%s\n', report, 'Input metadata file', ...
    s.input.metapath, s.input.metaname, s.input.metaformat);

report = sprintf('%s%s:  %s%s%s\n', report, 'Output file', ...
    s.output.path, s.output.recname, s.output.recformat);

end

