function [reconLog] = reconstructionRunner(data)
%UNTITLED4 Summary of this function goes here
%   Detailed explanation goes here

% reconRunner will choose the appropriate reconstruction algorithm based on
% settings in data.settings


% create a variable to store a text log in
reconLog = sprintf('%s\n', 'Attempting to reconstruct tomograms...');

try

    if data.settings.rec.recFlag
        % save start time (as time vector) to data.misc
        data.misc.reconStart = clock;
        % create a stopwatch to time how long the reconstruction took
        reconStopwatch = tic;
        
        % create a function handle based on the information in data.settings.rec
        reconFunctionString = ['reconstruct_', data.settings.rec.toolbox, ...
                                        '_', data.settings.rec.algorithm];
        
        reconFuncPath = which(reconFunctionString);
        
        if isempty(reconFuncPath)
            error('Reconstruction function not found.')
        else
            reconLog = sprintf('%s%s %s %s %s %s\n', reconLog, 'Initializing', ...
                data.settings.rec.algorithm, 'reconstruction using the', ...
                data.settings.rec.toolbox, 'toolbox');
            % create a function handle to the reconstruction function
            reconFunction = str2func(reconFunctionString);
        end
        
        % run function (should always have data as only input)
        [addToReconLog, saveMem] = reconFunction(data);
        % and it should always output a variable with text
        
        reconLog = sprintf('%s%s\n', reconLog, addToReconLog);
        
        % save time to data.misc
        data.misc.recTime = toc(reconStopwatch);
        % TODO - write it into reconstruction report
        % TODO - write into log
        
        reconLog = sprintf('%s%s: %.2f %s\n', reconLog, ...
                    'Total reconstruction time', data.misc.recTime, 's');
        
        % save tomograms and reconstruction report
        % TODO - save things I want to put into the report into data.misc
        if data.settings.output.recFlag
            
            % TODO - allow saving of preview, but add a prefix?
            [path, name, ~] = saveResult(data, 'rec', saveMem);
            % write to log
            reconLog = sprintf('%s%s\n', reconLog, 'Successfully saved tomograms');
            
            report = reconstructionReport(data);
            % TODO - save report inside reconstructionReport?
            fileID = fopen([path, name, '_report', '.txt'], 'w+');
            fprintf(fileID, '%s', report);
            fclose(fileID);
            
            reconLog = sprintf('%s%s\n%s\n', reconLog, ...
            'Reconstruction report created and saved to reconstruction output path.');
        end
        % TODO - write into log that report was created
        % TODO - save report
        
    else
        reconLog = sprintf('%s%s\n%s\n', reconLog, ...
            'Reconstruction flag set to false in data.settings.', ...
            'Aborting reconstruction.');
    end

catch err
    reconLog = sprintf('%s%s\n', reconLog, 'Unsuccessful in reconstructing data');
    reconLog = sprintf('%s%s\n', reconLog, getReport(err, 'extended', 'hyperlinks', 'off'));
    
    % if no output argument was specified, display the report as a warning
    if nargout == 0
        warning(reconLog)
    end
end

end

