function [path, name, format] = saveResult(data, type, saveMem)
%SAVERESULT saves either projection data or tomograms (according to type)
%   Detailed explanation goes here
% type - 'proj' or 'tomo'

% saveMem - whether memory should be conserved or not - logical

% TODO - description

% TODO - add precision as input?

if nargin < 3
    saveMem = false;
end


% determine the output path, name, and format
path = data.settings.output.path;
if strcmpi(type, 'proj')
    if data.settings.output.projFlag
        name = data.settings.output.projname;
        format = data.settings.output.projformat;
        field = 'proj';
    else % throw error if the flag is false
        error('outputStruct.projFlag set to false.')
    end
elseif strcmpi(type, 'rec')
    if data.settings.output.recFlag
        name = data.settings.output.recname;
        format = data.settings.output.recformat;
        field = 'tomo';
    else % throw error if the flag is false
        error('outputStruct.projFlag set to false.')
    end
end

% check if there isn't a file w/ the same name already
% also check for directories of that name (e.g. in case of saving .tiff)
if exist( [path, name, format] , 'file') || exist( [path, name] , 'dir')
    % if it is, try appending a number behind the filename
    k = 1;
    go = true;
    while go
        name_appended = [name, '_', num2str(k)];
        if exist( [path, name_appended, format] , 'file') || exist( [path, name_appended] , 'dir')
            % if that name is still taken, increment and try again
            k = k + 1;
        else
            % if the appended name is not taken, use that
            name = name_appended;
            go = false;
        end
    end
end

% get the data out of the object
data4saving = data.(field);
data.(field) = [];
% rotate the data (matlab is column-major, but data should be saved row by row)
if ~saveMem
    data4saving = permute(data4saving, [2 1 3]);
end


% TODO - probably can be done better

% proceeed to saving
if strcmp(format, '.raw')
    % saving the reconstruction as a *.raw file
    % open file in write mode (w+ just in case, but should be a new file)
    fileID = fopen([path, name, format], 'w+');
    fwrite(fileID, data4saving, 'uint16');
    fclose(fileID);
elseif strcmp(format, '.tiff') || strcmp(format, '.tif')
    % saving individual slices as *.tiff files
    % create a new folder for the images
    mkdir(path, name); 
    % get number of files in this folder (for when slices are saved in batches)
    dirInfo = dir([path, name]);
    existing_slices = length(dirInfo) - 2; % first 2 entries are just the directory
    % go through slices and save them one by one
    for sliceNum = 1:size(data4saving, 3)
        % create current slice's number, pad to 4 places w/ 0s
        slice_number = sprintf('%04i', sliceNum + existing_slices);
        % extract slice and convert to uint16
        slice = uint16( data4saving(:,:,sliceNum) );
        % save the slice
        imwrite( slice, ...
                [path, name, '\', slice_number, format]);
    end
    % TODO - can probably be done much better. Change if it causes trouble
    % TODO - check if parfor speeds this up at all
else
    error('%s: %s', 'Unsupported format', format)
end


% TODO - what to do with this ctm bit?

% load *.ctm file, overwrite
if strcmpi(type, 'proj')
    % load ctm file as text
    ctmFile = fileread([data.settings.input.metapath, ...
                        data.settings.input.metaname, ...
                        data.settings.input.metaformat]);
    
    % find info about projection width (pixels)
    [~,firstInd] = regexp(ctmFile, '<roi_dx_pixel>');
    [secondInd, ~] = regexp(ctmFile, '</roi_dx_pixel>');
    % overwrite projection width
    ctmFile = [ctmFile(1:firstInd), ...
        num2str(data.geom.width), ...
        ctmFile(secondInd:end)];
    
    % find info about projection width (mm)
    [~,firstInd] = regexp(ctmFile, '<roi_dx_mm>');
    [secondInd, ~] = regexp(ctmFile, '</roi_dx_mm>');
    % overwrite it, too
%     ctmFile = [ctmFile(1:firstInd), ...
%         num2str( (data.geom.width * data.geom.vox_size) / 1000), ...
%         ctmFile(secondInd:end)];
    ctmFile = [ctmFile(1:firstInd), ...
        num2str( (data.geom.width * data.geom.pix_size) / 1000), ...
        ctmFile(secondInd:end)];

    
    newCTM = fopen([path, name, '.ctm'], 'w');
    fprintf(newCTM, '%s', ctmFile);
    fclose(newCTM);
end

if ~saveMem
    % turn data back the way they were
    data4saving = permute(data4saving, [2 1 3]);
    
    % write back into projectionData object
    data.(field) = data4saving;
end

end

