close all; clear all; clc

initialization;

%% load data

data = projectionData;

% get path to projection data (and metadata)
[filename, inputpath] = uigetfile('*.raw', 'Choose RAW file containing projections', 'C:\');
dataPath = [inputpath, filename];

data.settings.addInput(dataPath);

% output
outputpath = uigetdir('C:\', 'Choose an output folder'); % choose folder
if outputpath ~= 0
    name = inputdlg('Enter output filename of data: ', 'Output filename'); % choose name
    
    % save info about saving projections and reconstruction as .raw
    data.settings.addOutput(outputpath, name{1}, '.raw', '.raw');
end

% create a projectionData object
% data.loadProj(1); % preview
data.loadProj(0); % full

% % change SOD and ODD values - TODO - do this?
% data.geom.edit('SOD', 264933) % um
% data.geom.edit('ODD', 1581 + (50 + data.params.XD_position)*1000) % um

% % original SOD and ODD values
% data.geom.edit('SOD', 275000) % um
% data.geom.edit('ODD', 7500) % um


if ~data.geom.offsetFlag % trim L2160 data
    
    data.proj = data.proj(626:1875, :, :);
    data.geom.edit('height', 1250, 'reconHeight', 1250)
    
end

%% motion correction

% [ xmotion, ymotion] = Grad_CrossCorr( double(data.proj(:,:,1)), ...
%     double(data.proj(:,:,end)) );

% xmotion = -1;
% ymotion = 0;
% 
% % Motion and axis correction - TODO - check
% x_shift = cos(data.geom.angles).* linspace(0, xmotion, data.geom.num_proj);
% y_shift = linspace(0, ymotion, data.geom.num_proj);
% % shifting each projection according to its estimated motion before trimming it
% for ii = 1:data.geom.num_proj
%     
%     proj(:, :, ii) = imtranslate(data.proj(:, :, ii), ...
%                                  [x_shift(ii), y_shift(ii)], ...
%                                  'OutputView', 'same', ...
%                                  'method', 'bicubic', ...
%                                  'FillValues', mean( mean(data.proj(:, :, ii)) ) );
% end
% data.proj = proj;

%% remove excess projections

if data.geom.offsetFlag % L1080 optics
    % remove 1st and last proj
    data.proj = data.proj(:,:, 2:end-1);
    data.geom.edit('angles', data.geom.angles(2:end-1));
    
else % L2160 optics
    
    % remove 1st proj
    data.proj = data.proj(:,:, 2:end);
    data.geom.edit('angles', data.geom.angles(2:end));
    
end


%% AoR estimation

corEstimManual(data) % see how good the data are

% corUserInput(data, 149);

% corEstimRec(data, 1, 1);
% corOffsetSino(data, 1);


%% reconstruct

% settings
if data.geom.offsetFlag
    data.settings.addTask( 'virtualDetTransf', 'keepVoxelSize', true );
    data.settings.addTask( 'trans2absorp' );
    data.settings.addTask( 'offsetWeighPad', 'padFlag', true ); % only for FDK
    data.settings.addTask( 'truncationPadding', 'padding', 10 ); % only for FDK
    % data.settings.addTask( 'makeProjWidthEven');
    % data.settings.addTask( 'cuppingReduction');
    data.settings.addRec('ASTRA', 'FDK', ...
             'filterType', 'cosine', 'freqCutoff', 0.85, 'limitGPU', true);
else
    data.settings.addTask( 'trans2absorp' );
    data.settings.addTask( 'truncationPadding', 'padding', 10 ); % only for FDK
    data.settings.addRec('ASTRA', 'FDK', ...
             'filterType', 'cosine', 'freqCutoff', 0.85, 'limitGPU', true);
end

%% run
taskLog = taskRunner(data);
reconLog = reconstructionRunner(data);











