close all; clear; clc

initialization;

%% Set all parameters

% 'shepp-logan', 'original shepp', 'siemens star' 1, 'circle',
% 'concentric parabolic' 1 'concentric linear' 1, 'circles' 1, 'point',
% 'grid' 1, 'squares' 1, 'resolution' 1

set = struct;
set.det_count = 1648; % number of detector bins AFTER BINNING
set.extension = 1.8; % 1.8;
set.pix_size_b1 = 1.08; % pixel pitch at binning 1
set.binning = 2; % if this is e.g. 2, then the input data with width N gets subsampled by 1/2;
set.SOD = 275000;
set.ODD = 7500;
set.num_proj = 1600;
set.angleRange = 2*pi;

% set.incident_flux = 5e5;
% set.poissonNoise = false;
% set.gaussianNoise = 0;
% set.impulseNoise = 0;
% set.rings = 0; % ring artifacts
% set.exp_decrease = 1;
% set.transmissionOutput = true;
% set.fullSino = false; % overrides set.det_count

set.incident_flux = 50000;
set.poissonNoise = true;
set.gaussianNoise = 100;
set.impulseNoise = 0.000005;
set.rings = 0.001; % ring artifacts
set.exp_decrease = 0.95;
set.transmissionOutput = true;
set.fullSino = false; % overrides set.det_count

phantomName = 'shepp-logan';
set.phantomOption = 5; % some phantoms allow for a third input. these are marked by a 1 above
set.outputBinnedPhantom = true;

%% Create phantom, get sinogram

% make offset data
[data, out] = makeSyntheticSino(phantomName, set);
% make full data
set.fullSino = true;
[dataFull] = makeSyntheticSino(phantomName, set);

%% Save data

% output
outputpath = uigetdir('C:\', 'Choose an output folder (cancel if you don''t want to save the results)'); % choose folder
if outputpath ~= 0
    name = inputdlg('Enter output filename of data: ', 'Output filename'); % choose name
    
    % save info about saving projections and reconstruction as .raw
    data.settings.addOutput(outputpath, [name{1} 'Offset'], '.raw', '.raw');
    dataFull.settings.addOutput(outputpath, [name{1} 'Full'], '.raw', '.raw');
end

%% Try things out

CoRRec = corEstimRec(data, 1, 1);
CoRSino = corOffsetSino(data, 1);
% CoR = corEstimManual(data);

corInput(data, 165)

%% Setup reconstruction

% SETTINGS FOR OFFSET DATA
data.settings.rmAllTasks;
% data.settings.addTask( 'corEstimRec', 'slicesToAverage', 1, 'broadSearch', 1);
% data.settings.addTask( 'corOffsetSino', 'slicesToAverage', 1);
% data.settings.addTask( 'corEstimManual');
data.settings.addTask( 'virtualDetTransf', 'keepVoxelSize', true );
data.settings.addTask( 'trans2absorp' );
data.settings.addTask( 'offsetWeighPad', 'padFlag', true ); % only for FDK
data.settings.addTask( 'truncationPadding', 'padding', 10 ); % only for FDK
% data.settings.addTask( 'makeProjWidthEven');
% data.settings.addTask( 'cuppingReduction');
data.settings.addRec('ASTRA', 'FDK', ...
         'filterType', 'cosine', 'freqCutoff', 0.85, 'limitGPU', true);
     
% SETTIGNS FOR REGULAR DATA
dataFull.settings.rmAllTasks;
% not adding CoR estimation - I know it's not shifted
% data.settings.addTask( 'corOffsetRec', 'slicesToAverage', 1, 'broadSearch', 1);
% data.settings.addTask( 'corOffsetSino', 'slicesToAverage', 1);
dataFull.settings.addTask( 'trans2absorp' );
dataFull.settings.addTask( 'truncationPadding', 'padding', 10 ); % only for FDK
dataFull.settings.addRec('ASTRA', 'FDK', ...
         'filterType', 'cosine', 'freqCutoff', 0.85, 'limitGPU', true);

%% Run tasks and reconstruction
% taskLog = taskRunner(data, [1 2 3 4 5]);
taskLog = taskRunner(data);
reconLog = reconstructionRunner(data);

taskLogFull = taskRunner(dataFull);
reconLogFull = reconstructionRunner(dataFull);

if outputpath ~= 0
    savePhantom(name{1}, outputpath, out, set);
end


% normalize reconstructions (ground truth is in the range 0-1)
data.tomo = (data.tomo - min(data.tomo(:))) / (max(data.tomo(:)) - min(data.tomo(:)));
dataFull.tomo = (dataFull.tomo - min(dataFull.tomo(:))) / (max(dataFull.tomo(:)) - min(dataFull.tomo(:)));

figure; imshow(data.tomo, [] ); title('Offset data')
figure; imshow(dataFull.tomo, [] ); title('Full data')

% difference of full rec and offset rec
figure; imshow( dataFull.tomo - data.tomo, [] ); title('difference between full and offset')

% difference of full rec and ground truth 
figure; imshow( out.binnedPhantom - data.tomo, [] ); title('difference between ground truth and offset data')
% TODO - make it so ground truth is the same size
figure; imshow( out.binnedPhantom - dataFull.tomo, [] ); title('difference between ground truth and full data')

%% Evaluate

% normalize reconstructions (ground truth is in the range 0-1)
data.tomo = (data.tomo - min(data.tomo(:))) / (max(data.tomo(:)) - min(data.tomo(:)));
dataFull.tomo = (dataFull.tomo - min(dataFull.tomo(:))) / (max(dataFull.tomo(:)) - min(dataFull.tomo(:)));

[X, Y] = meshgrid( linspace(-1, 1, out.N) );
mask = (X.^2 + Y.^2) <= 1;

data.tomo(~mask) = 0;
dataFull.tomo(~mask) = 0;

% MSE
MSEfull = immse(out.binnedPhantom(mask), dataFull.tomo(mask));
MSEoffset = immse(out.binnedPhantom(mask), data.tomo(mask));

% PSNR
PSNRfull = 20 * log10( 1 / sqrt(MSEfull) );
PSNRoffset = 20 * log10( 1 / sqrt(MSEoffset) );

% FSIM
FSIMfull = FeatureSIM(out.binnedPhantom.*255, dataFull.tomo.*255); % dynamic range should be 0-255
FSIMoffset = FeatureSIM(out.binnedPhantom.*255, data.tomo.*255); % dynamic range should be 0-255

if outputpath ~= 0
    fID = fopen([outputpath, '\eval_', name{1}, '.txt'], 'w+');
    fprintf(fID, ['%s\r\nMSE full: %0.4f\r\nMSE offset: %0.4f\r\n' ...
                   'PSNR full: %0.4f\r\nPSNR offset: %0.4f\r\n' ...
                   'FSIM full: %0.4f\r\nFSIM offset: %0.4f\r\n'], ...
                   name{1}, MSEfull, MSEoffset, PSNRfull, PSNRoffset, ...
                   FSIMfull, FSIMoffset);
    fclose(fID);
end

