function [CoR] = corInput(data, CoR)
%CORUSERINPUT is for direct input of CoR estimation of offset-sample data
%   This function creates a simple input GUI for inputing the position of
%   the Center of rotation in offset data. As an alternative, the CoR
%   estimate can be supplied to this function as the second input This
%   may be useful is the CoR estimate is already known.
%   Inputs:
%       data - projectionData object
%       CoR - scalar, the detector bin at which the center of rotation is
%       (optional input)
%   Outputs:
%       CoR - estimate of the location of the CoR (as the index of a
%       detector bin). This CoR estimate is also written into the GEOM 
%       property of data (scalar)
%   Example:
%       CoR = corUserInput(data, 45);
%       CoR = corUserInput(data);
%       corUserInput(data);

% TODO - check inputs
% TODO - make more robust

if nargin == 1 % if a CoR estimate is not supplied, open dialog for input
    prompt = {'Enter the index of the pixel at the center of rotation'};
    dlgtitle = 'CoR user input';
    results = inputdlg(prompt, dlgtitle);
    if ~isempty(results)
        CoR = str2double(results{1});
    end
end

% TODO - use try/catch or nah?
try % check if the CoR estimate is alright
    if CoR < 1
        CoR = 1;
%     elseif CoR > (data.geom.width + 1)/2
%         CoR = (data.geom.width + 1)/2;
    end
catch % if anything went wrong, just output the old CoR
    CoR = data.geom.CoR;
end

% write new values into data.geom
data.geom.edit('CoR', CoR);

if data.geom.offsetFlag % change size of reconstruction only if it is an offset scan
    extendedReconWidth = round( (data.geom.width - CoR ) * 2);
    data.geom.edit('reconWidth', extendedReconWidth);
end

end

