function truncationPadding(data, padding)
%TRUNCATIONPADDING pads projection data to reduce the truncation artifact
%   This function takes projection data from the projectionData object and
%   pads them on either side to suppress truncation artifacts. The padded
%   region on either side is of length "padding" and consists of the last
%   measured sinogram row weighed by a linear function going from 1 to 0.
% Inputs:
%   data - PROJECTIONDATA object containing projections to be padded (in
%   the PROJ property
%   padding - positive whole scalar signifying the number of pixels to pad
%   the projections by on each side
% Example:
%   truncationPadding(data, 10)

% a pad of about 10 pixels on either side is usually sufficient
% padding consists of the last measured sinogram pixel and gradually goes to 0
pad_left = repmat(data.proj(:,1,:), 1, padding, 1) .* linspace(0, 1, padding);
pad_right = repmat(data.proj(:,end,:), 1, padding, 1) .* linspace(1, 0, padding);
data.proj = [pad_left, data.proj, pad_right];

% change info about width in geom. This also shifts CoR by padding
data.geom.edit('width', size(data.proj, 2), ...
               'CoR', data.geom.CoR + padding);
% reconstruction width doesn't change
end

