function cyllinderTrim(data, outsideVal)
%CYLLINDERTRIM removes values outside the cyllindrical reconstruction FoV
%   this function takes a reconstructed CT volume, creates a cyllindrical
%   mask of the same size, and sets values outside this mask to the value
%   OUTSIDEVAL (default 0). The reconstructed CT volume is in the TOMO
%   field of the PROJECTIONDATA object.
%   Inputs:
%       data - projectionData object
%       outsideVal - scalar, the value to write into pixels outside the FoV
%   Example:
%       cyllinderTrim(data);
%       cyllinderTrim(data, 1000);
%       cyllinderTrim(data, mean( data.proj(:) ) );

% if no outsideVal was supplied, set it to 0 by default
if nargin < 2
    outsideVal = 0;
end

% get recosntructiion size (independent on data object)
width = size(data.tomo, 1);
height = size(data.tomo, 3);
% create x-y meshgrid
[x ,y] = meshgrid(1:width, ...
                  1:width); 
% create 2D circular mask
mask = sqrt((x - (width + 1)/2).^2 + ...
            (y - (width + 1)/2).^2) ...
         > (width/2);
     
maskHeight = min(height, 20);
mask = repmat(mask, 1, 1, maskHeight);
     
% % use mask on data - in chunks
tomograms = data.tomo;
data.tomo = [];
for k = 1:maskHeight:height
    
    if (k + maskHeight - 1) > height
        extent = height;
        maskExtent = height - k + 1;
    else
        extent = k + maskHeight - 1;
        maskExtent = maskHeight;
    end
    
    singleTomo = tomograms(:,:,k:extent);
    singleTomo( mask(:,:, 1:maskExtent) ) = outsideVal;
    tomograms(:,:,k:extent) = singleTomo; % use mask on data
end % todo - check
data.tomo = tomograms;

end

